/* This file is part of the KDE project
   Copyright (C) 2001, The Karbon Developers
   Copyright (C) 2002, The Karbon Developers

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
*/


#include "vcomposite.h"
#include "vdocument.h"
#include "vselection.h"
#include "vshapecmd.h"


VShapeCmd::VShapeCmd( VDocument* doc, const TQString& name, VPath* tqshape, const TQString& icon )
	: VCommand( doc, name, icon ), m_tqshape( tqshape )
{
}

void
VShapeCmd::execute()
{
	if( !m_tqshape )
		return;

	if( m_tqshape->state() == VObject::deleted )
	{
		document()->selection()->clear();
		m_tqshape->setState( VObject::normal );
		document()->selection()->append( m_tqshape );
	}
	else
	{
		m_tqshape->setState( VObject::normal );
		m_tqshape->setFill( *( document()->selection()->fill() ) );
		m_tqshape->setStroke( *( document()->selection()->stroke() ) );

		// Add path:
		document()->append( m_tqshape );
		document()->selection()->clear();
		document()->selection()->append( m_tqshape );
	}

	setSuccess( true );
}

void
VShapeCmd::unexecute()
{
	if( !m_tqshape )
		return;

	document()->selection()->take( *m_tqshape );
	m_tqshape->setState( VObject::deleted );

	setSuccess( false );
}

