/* This file is part of the KDE project
   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
*/

#ifndef KEXIGUIMSGHANDLER_H
#define KEXIGUIMSGHANDLER_H

#include <core/kexi.h>
#include <kexidb/msghandler.h>

class KEXICORE_EXPORT KexiGUIMessageHandler : public KexiDB::MessageHandler
{
	public:
		KexiGUIMessageHandler(TQWidget *parent = 0);
		virtual ~KexiGUIMessageHandler();
		virtual void showErrorMessage(const TQString &title, const TQString &details = TQString());
		virtual void showErrorMessage(KexiDB::Object *obj, const TQString& msg = TQString());

		void showErrorMessage(const TQString&,const TQString&,KexiDB::Object *obj);
		void showErrorMessage(Kexi::ObjecttqStatus *status);
		void showErrorMessage(const TQString &message, Kexi::ObjecttqStatus *status);

		/*! Displays a "Sorry" message with \a title text and optional \a details. */
		void showSorryMessage(const TQString &title, const TQString &details = TQString());

		/*! Displays a message of a type \a type, with \a title text and optional \a details.
		 \a dontShowAgainName can be specified to add "Do not show again" option if \a type is Warning. */
		virtual void showMessage(MessageType type, const TQString &title, const TQString &details,
			const TQString& dontShowAgainName = TQString());

		/*! Displays a Warning message with \a title text and optional \a details 
		 with "Continue" button instead "OK".
		 \a dontShowAgainName can be specified to add "Do not show again" option. */
		virtual void showWarningContinueMessage(const TQString &title, const TQString &details = TQString(),
			const TQString& dontShowAgainName = TQString());

		/*! Interactively asks a question using KMessageBox.
		 See KexiDB::MessageHandler::askQuestion() for details. */
		virtual int askQuestion( const TQString& message, 
			KMessageBox::DialogType dlgType, KMessageBox::ButtonCode defaultResult,
			const KGuiItem &buttonYes=KStdGuiItem::yes(), 
			const KGuiItem &buttonNo=KStdGuiItem::no(),
			const TQString &dontShowAskAgainName = TQString(),
			int options = KMessageBox::Notify );
};

#endif
