/* This file is part of the KDE project
   Copyright (C) 2005 Cedric Pasteur <cedric.pasteur@free.fr>
   Copyright (C) 2004-2005 Jaroslaw Staniek <js@iidea.pl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
*/

#ifndef KexiDBDateEdit_H
#define KexiDBDateEdit_H

#include "kexiformdataiteminterface.h"
#include <tqdatetimeedit.h>

class KPopupMenu;
class KDatePicker;
class TQDateTimeEditor;

//! @short A db-aware date editor
class KEXIFORMUTILS_EXPORT KexiDBDateEdit : public TQWidget, public KexiFormDataItemInterface
{
	Q_OBJECT
  TQ_OBJECT
	TQ_PROPERTY(TQString dataSource READ dataSource WRITE setDataSource DESIGNABLE true)
	TQ_PROPERTY(TQCString dataSourceMimeType READ dataSourceMimeType WRITE setDataSourceMimeType DESIGNABLE true)
	// properties copied from TQDateEdit
	Q_ENUMS( Order )
	TQ_PROPERTY( Order order READ order WRITE setOrder DESIGNABLE true)
	TQ_PROPERTY( TQDate date READ date WRITE setDate DESIGNABLE true)
	TQ_PROPERTY( bool autoAdvance READ autoAdvance WRITE setAutoAdvance DESIGNABLE true)
	TQ_PROPERTY( TQDate maxValue READ maxValue WRITE setMaxValue DESIGNABLE true)
	TQ_PROPERTY( TQDate minValue READ minValue WRITE setMinValue DESIGNABLE true)
	TQ_PROPERTY( bool readOnly READ isReadOnly WRITE setReadOnly DESIGNABLE true )

	public:
		enum Order { DMY = TQDateEdit::DMY, MDY = TQDateEdit::MDY, YMD = TQDateEdit::YMD,  YDM = TQDateEdit::YDM };

		KexiDBDateEdit(const TQDate &date, TQWidget *parent, const char *name=0);
		virtual ~KexiDBDateEdit();

		inline TQString dataSource() const { return KexiFormDataItemInterface::dataSource(); }
		inline TQCString dataSourceMimeType() const { return KexiFormDataItemInterface::dataSourceMimeType(); }
		virtual TQVariant value();
		virtual void setInvalidState( const TQString& displayText );

		//! \return true if editor's value is null (not empty)
		//! Used for checking if a given constraint within table of form is met.
		virtual bool valueIsNull();

		//! \return true if editor's value is empty (not necessary null).
		//! Only few data types can accept "EMPTY" property
		//! (use KexiDB::Field::hasEmptyProperty() to check this).
		//! Used for checking if a given constraint within table or form is met.
		virtual bool valueIsEmpty();

		/*! \return 'readOnly' flag for this widget. */
		virtual bool isReadOnly() const;

		/*! \return the view widget of this item, e.g. line edit widget. */
		virtual TQWidget* widget();

		virtual bool cursorAtStart();
		virtual bool cursorAtEnd();
		virtual void clear();

		virtual void  setEnabled(bool enabled);

		// property functions
		inline TQDate date() const { return m_edit->date(); }
		inline void setOrder(Order order) { m_edit->setOrder( (TQDateEdit::Order) order); }
		inline Order order() const { return (Order)m_edit->order(); }
		inline void setAutoAdvance( bool advance ) { m_edit->setAutoAdvance(advance); }
		inline bool autoAdvance() const { return m_edit->autoAdvance(); }
		inline void setMinValue(const TQDate& d) { m_edit->setMinValue(d); }
		inline TQDate minValue() const { return m_edit->minValue(); }
		inline void setMaxValue(const TQDate& d) { m_edit->setMaxValue(d); }
		inline TQDate maxValue() const { return m_edit->maxValue(); }

	signals:
		void  dateChanged(const TQDate &date);

	public slots:
		inline void setDataSource(const TQString &ds) { KexiFormDataItemInterface::setDataSource(ds); }
		inline void setDataSourceMimeType(const TQCString &ds) { KexiFormDataItemInterface::setDataSourceMimeType(ds); }
		inline void setDate(const TQDate& date)  { m_edit->setDate(date); }
		virtual void setReadOnly(bool set);

	protected slots:
		void slotValueChanged(const TQDate&);
		void  slotShowDatePicker();
		void  acceptDate();

	protected:
		virtual void setValueInternal(const TQVariant& add, bool removeOld);
		virtual bool  eventFilter(TQObject *o, TQEvent *e);

	private:
		KDatePicker *m_datePicker;
		TQDateEdit *m_edit;
		KPopupMenu *m_datePickerPopupMenu;
		TQDateTimeEditor *m_dte_date;
		bool m_invalidState : 1;
		bool m_cleared : 1;
		bool m_readOnly : 1;
};

#endif
