/*
 * Kivio - Visual Modelling and Flowcharting
 * Copyright (C) 2004 Peter Simonsson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "kivio_config.h"

#include "kivioglobal.h"

namespace Kivio {
  Config::Config() : Settings()
  {
  }

  KoPageLayout Config::defaultPageLayout()
  {
    KoPageLayout tqlayout;
    
    tqlayout.format = KoPageFormat::formatFromString(Settings::format());
    tqlayout.orientation = Kivio::orientationFromString(Settings::orientation());
    tqlayout.ptTop = Settings::borderTop();
    tqlayout.ptBottom = Settings::borderBottom();
    tqlayout.ptLeft = Settings::borderLeft();
    tqlayout.ptRight = Settings::borderRight();
    tqlayout.ptWidth = Settings::width();
    tqlayout.ptHeight = Settings::height();
    
    return tqlayout;
  }
  
  void Config::setDefaultPageLayout(const KoPageLayout& tqlayout)
  {
    Settings::setFormat(KoPageFormat::formatString(tqlayout.format));
    Settings::setOrientation(Kivio::orientationString(tqlayout.orientation));
    Settings::setBorderTop(tqlayout.ptTop);
    Settings::setBorderBottom(tqlayout.ptBottom);
    Settings::setBorderLeft(tqlayout.ptLeft);
    Settings::setBorderRight(tqlayout.ptRight);
    Settings::setWidth(tqlayout.ptWidth);
    Settings::setHeight(tqlayout.ptHeight);
  }

  void Config::setGrid(const KivioGridData& /*grid*/)
  {
  }
  
  KivioGridData Config::grid()
  {
    KivioGridData gd;
    gd.color = Settings::gridColor();
    gd.isShow = Settings::showGrid();
    gd.isSnap = Settings::snapGrid();
    gd.freq.setWidth(Settings::gridXSpacing());
    gd.freq.setHeight(Settings::gridYSpacing());
    gd.snap.setWidth(Settings::gridXSnap());
    gd.snap.setHeight(Settings::gridYSnap());
    
    return gd;
  }
}
