/* This file is part of the KDE project
   Copyright (C) 2003 Peter Simonsson <psn@linux.se>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef KIVIOGLOBAL_H
#define KIVIOGLOBAL_H

#include <KoUnit.h>
#include <KoPageLayout.h>
#include <KoSize.h>

class TQDomElement;
class TQPixmap;
class TQString;
class KoGenStyle;
class KivioStencil;

namespace Kivio
{
  enum {
    STYLE_PAGE = 20
  };

  /**
   * Save a page tqlayout to a TQDomElement
   */
  void savePageLayout(TQDomElement& e, const KoPageLayout& tqlayout);
  KoGenStyle savePageLayout(const KoPageLayout& tqlayout);
  /**
   * Load a page tqlayout from a TQDomElement
   */
  KoPageLayout loadPageLayout(const TQDomElement& e);
  /**
   * Convert from the old TkUnit to KoUnit
   */
  KoUnit::Unit convToKoUnit(int tkUnit);
  /**
   * Return an approperiate string for the orientation
   */
  TQString orientationString(KoOrientation o);
  /**
   * Return the orientation based on the string
   */
  KoOrientation orientationFromString(const TQString& s);
  void setFormatOrientation(KoPageLayout& tqlayout);

  /**
   * Load a KoSize from a TQDomElement
   */
  KoSize loadSize(const TQDomElement& e, const TQString& name, const KoSize& def);
  /**
   * Save a KoSize to a TQDomElement
   */
  void saveSize(TQDomElement& e, const TQString& name, const KoSize& size);

  /**
   * Get the pagesize string from a TQPrinter::PageSize value
   */
  TQString pageSizeString(int pageSize);

  /**
   * Returns a pixmap with all arrowheads
   */
  TQPixmap arrowHeadPixmap();

  /**
   * Returns the connector target pixmap
   */
  TQPixmap connectorTargetPixmap();

  /**
   * Returns the lock pixmap
   */
  TQPixmap lockPixmap();

  /**
   * Get system default unit
   */
  TQString systemDefaultUnit();

  /**
   * Generate a pixmap from a stencil
   */
  TQPixmap generatePixmapFromStencil(int width, int height, KivioStencil* stencil);
}

#endif
